@@ -27,13 +27,19 @@ History:
                                                              -added mask to getInnerOuterMeanDiff, Simplified_Mumford_Shah_driver,and initSnakeStack
           w.x.chan@gmail.com         19Jan2021           - v2.7.14
                                                              -added manualSliceBySlice
+          w.x.chan@gmail.com         20Jan2021           - v2.7.17
+                                                             -debug manualSliceBySlice to get smooth interpolation between segmented slice
+          w.x.chan@gmail.com         25Jan2021           - v2.7.19
+                                                             -debug manualSliceBySlice
+          w.x.chan@gmail.com         25Jan2021           - v2.7.20
+                                                             -in manualSliceBySlice, reorder lines and debug multiple lines in slice
 Requirements:
     numpy
 Known Bug:
     None
 All rights reserved.
 '''
-_version='2.7.14'
+_version='2.7.20'
 import logging
 logger = logging.getLogger(__name__)
 
@@ -43,6 +49,8 @@ import numpy as np
 from scipy.ndimage import morphology
 from scipy.ndimage import gaussian_filter
 from scipy.ndimage import laplace
+from scipy import interpolate
+from scipy.ndimage import binary_fill_holes
 try:
     from skimage import measure
 except:
@@ -490,10 +498,24 @@ def initSnakeStack(imageArray,snakeInitCoordList,driver=None,initSize=1,setSnake
     driver.snakeStackClass=resultSnakeStack
     resultSnakeStack.check()
     return resultSnakeStack
-    
-        
-def manualSliceBySlice(img):
-    a=img.show(disable['click','swap'])
+
+def manualSliceBySlice(img,initLineList=None,lengthScaleRatio=0.2):
+    if initLineList is not None:
+        if isinstance(initLineList,list) and len(initLineList[0].shape)==2:
+            a=img.show(disable=['click','swap'],initLineList=initLineList)
+        else:
+            newInitLineList=[[]]
+            currentslice=initLineList[0][0]
+            for n in range(len(initLineList)):
+                if currentslice!=initLineList[n][0]:
+                    newInitLineList.append([])
+                    currentslice=initLineList[n][0]
+                newInitLineList[-1].append(initLineList[n])
+            for n in range(len(newInitLineList)):
+                newInitLineList[n]=np.array(newInitLineList[n])
+            a=img.show(disable=['click','swap'],initLineList=newInitLineList)
+    else:
+        a=img.show(disable=['click','swap'])
     while not(a.enter):
         a=img.show(disable=['click','swap'],initLineList=a.lines)
     for n in range(len(a.lines)-1,-1,-1):
@@ -505,36 +527,51 @@ def manualSliceBySlice(img):
     else:
         if len(img.dim)>3:
             raise Exception('Dimension of image more than 3,'+str(img.dim))
+    getind=[]
+    for n in range(len(a.lines)):
+        getind.append(a.lines[n][0][0])
+    getind=np.array(getind).astype(int)
+    a.lines = [a.lines[i] for i in np.sort(getind)]
     aind=0
     img2=img.clone()
     if a.color:
         img2.changeGreyscaleFormat()
     img2.data[:]=0
     img2.data=img2.data.astype('uint8')
+    minArea=float('inf')
     for n in range(img2.data.shape[0]):
-        if a.lines[aind][0][0]==n:
-            ar=np.array(a.lines[aind])
-            tck,temp = interpolate.splprep([ar[:,-2], ar[:,-1]], s=0,k=min(4,len(ar))-1)
-            cspline_detectline = np.array(interpolate.splev(np.linspace(0, 1,num=1000), tck)).T
-            cspline_detectline2=np.floor(cspline_detectline).astype(int)
-            for nn in range(len(cspline_detectline)):
-                img2.data[n][tuple(cspline_detectline[nn])]=1
-            ar2=np.array([ar[0],0.5*(ar[0]+ar[-1]),ar[-1]])
-            tck,temp = interpolate.splprep([ar2[:,-2], ar2[:,-1]], s=0,k=1)
-            cspline_detectline = np.array(interpolate.splev(np.linspace(0, 1,num=1000), tck)).T
-            cspline_detectline=np.around(cspline_detectline).astype(int)
-            for nn in range(len(cspline_detectline)):
-                img2.data[n][tuple(cspline_detectline[nn])]=1
-            img2.data[n]=binary_fill_holes(img2.data[n]).astype(img2.data.dtype)
+        while a.lines[aind][0][0]==n:
+            for num in [1000,10000,100000,1000000]:
+                ar=np.array(a.lines[aind])
+                tck,temp = interpolate.splprep([ar[:,-2], ar[:,-1]], s=0,k=min(4,len(ar))-1)
+                cspline_detectline = np.array(interpolate.splev(np.linspace(0, 1,num=num), tck)).T
+                cspline_detectline=np.floor(cspline_detectline).astype(int)
+                for nn in range(len(cspline_detectline)):
+                    img2.data[n][tuple(cspline_detectline[nn])]=1
+                ar2=np.array([ar[0],0.5*(ar[0]+ar[-1]),ar[-1]])
+                tck,temp = interpolate.splprep([ar2[:,-2], ar2[:,-1]], s=0,k=1)
+                cspline_detectline = np.array(interpolate.splev(np.linspace(0, 1,num=num), tck)).T
+                cspline_detectline=np.floor(cspline_detectline).astype(int)
+                for nn in range(len(cspline_detectline)):
+                    img2.data[n][tuple(cspline_detectline[nn])]=1
+                countpixel=img2.data[n].sum()
+                img2.data[n]=binary_fill_holes(img2.data[n]).astype(img2.data.dtype)
+                if img2.data[n].sum()>(countpixel*1.1):
+                    break
+            else:
+                logger.warning('Could not form a closed surface with area larger than 1.1 of boundary on Slice '+str(n))
+            temp_area=img2.data[n].sum()
+            if temp_area<minArea:
+                minArea=temp_area
             aind+=1
             if aind>=len(a.lines):
                 break
+        if aind>=len(a.lines):
+            break
     img2.data*=255
+    lengthScale=max(2,lengthScaleRatio*minArea**0.5)
+    img2.data=gaussian_filter(img2.data,(0,lengthScale,lengthScale))
     img3=img2.clone()
-    getind=[]
-    for n in range(len(a.lines)):
-        getind.append(a.lines[n][0][0])
-    getind=np.array(getind).astype(int)
     currentind=0
     for n in range(getind[0]+1,getind[-1]):
         if n==getind[currentind+1]:
@@ -548,11 +585,9 @@ def manualSliceBySlice(img):
     else:
         nDim=[0,-2,-1]
     for n in nDim:
-        sigma.append(img3.dimlen[dim])
-    sigma=np.array(sigma)
-    sigma
-    img3.data=gaussian_filter(img3.data,)
-    return img3
+        sigma.append(img3.dimlen[img3.dim[n]])
+    sigma=1./np.array(sigma)
+    sigma=sigma/sigma[1:].mean()*lengthScale
+    img3.data=gaussian_filter(img3.data,sigma)
+    return (a.lines,img3)
 
-    
-    
